/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "tyon_polling_rate_selector.h"
#include "roccat_polling_rate_selector.h"
#include "tyon_profile_settings.h"

#define TYON_POLLING_RATE_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), TYON_POLLING_RATE_SELECTOR_TYPE, TyonPollingRateSelectorClass))
#define IS_TYON_POLLING_RATE_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TYON_POLLING_RATE_SELECTOR_TYPE))
#define TYON_POLLING_RATE_SELECTOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), TYON_POLLING_RATE_SELECTOR_TYPE, TyonPollingRateSelectorPrivate))

typedef struct _TyonPollingRateSelectorClass TyonPollingRateSelectorClass;

struct _TyonPollingRateSelector {
	RoccatPollingRateSelector parent;
};

struct _TyonPollingRateSelectorClass {
	RoccatPollingRateSelectorClass parent_class;
};

G_DEFINE_TYPE(TyonPollingRateSelector, tyon_polling_rate_selector, ROCCAT_POLLING_RATE_SELECTOR_TYPE);

GtkWidget *tyon_polling_rate_selector_new(void) {
	TyonPollingRateSelector *polling_rate_selector;

	polling_rate_selector = TYON_POLLING_RATE_SELECTOR(g_object_new(TYON_POLLING_RATE_SELECTOR_TYPE,
			"mask", ROCCAT_POLLING_RATE_SELECTOR_ALL,
			NULL));

	return GTK_WIDGET(polling_rate_selector);
}

static void tyon_polling_rate_selector_init(TyonPollingRateSelector *polling_rate_selector) {}

static void tyon_polling_rate_selector_class_init(TyonPollingRateSelectorClass *klass) {}

void tyon_polling_rate_selector_set_value(TyonPollingRateSelector *polling_rate_selector, guint rmp_value) {
	guint roccat_value;

	switch (rmp_value) {
	case TYON_PROFILE_SETTINGS_POLLING_RATE_125:
		roccat_value = ROCCAT_POLLING_RATE_SELECTOR_125;
		break;
	case TYON_PROFILE_SETTINGS_POLLING_RATE_250:
		roccat_value = ROCCAT_POLLING_RATE_SELECTOR_250;
		break;
	case TYON_PROFILE_SETTINGS_POLLING_RATE_500:
		roccat_value = ROCCAT_POLLING_RATE_SELECTOR_500;
		break;
	case TYON_PROFILE_SETTINGS_POLLING_RATE_1000:
		roccat_value = ROCCAT_POLLING_RATE_SELECTOR_1000;
		break;
	default:
		g_warn_if_reached();
		roccat_value = ROCCAT_POLLING_RATE_SELECTOR_NONE;
		break;
	}

	roccat_polling_rate_selector_set_value(ROCCAT_POLLING_RATE_SELECTOR(polling_rate_selector), roccat_value);
}

guint tyon_polling_rate_selector_get_value(TyonPollingRateSelector *polling_rate_selector) {
	guint roccat_value;
	guint rmp_value;

	roccat_value = roccat_polling_rate_selector_get_value(ROCCAT_POLLING_RATE_SELECTOR(polling_rate_selector));

	switch (roccat_value) {
	case ROCCAT_POLLING_RATE_SELECTOR_125:
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_125;
		break;
	case ROCCAT_POLLING_RATE_SELECTOR_250:
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_250;
		break;
	case ROCCAT_POLLING_RATE_SELECTOR_500:
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_500;
		break;
	case ROCCAT_POLLING_RATE_SELECTOR_1000:
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_1000;
		break;
	case ROCCAT_POLLING_RATE_SELECTOR_NONE:
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_1000;
		// TODO warning
		break;
	default:
		g_warn_if_reached();
		rmp_value = TYON_PROFILE_SETTINGS_POLLING_RATE_1000;
		break;
	}

	return rmp_value;
}
