/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "tyon_sensitivity_selector.h"
#include <gaminggear/gaminggear_xy_scales.h>

#define TYON_SENSITIVITY_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), TYON_SENSITIVITY_SELECTOR_TYPE, TyonSensitivitySelectorClass))
#define IS_TYON_SENSITIVITY_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TYON_SENSITIVITY_SELECTOR_TYPE))

typedef struct _TyonSensitivitySelectorClass TyonSensitivitySelectorClass;

struct _TyonSensitivitySelector {
	GaminggearXyScales parent;
};

struct _TyonSensitivitySelectorClass {
	GaminggearXyScalesClass parent_class;
};

G_DEFINE_TYPE(TyonSensitivitySelector, tyon_sensitivity_selector, GAMINGGEAR_XY_SCALES_TYPE);

static guint scale_to_bin(gdouble scale) {
	return (guint)(scale + 6.0);
}

static gdouble bin_to_scale(guint bin) {
	return (gdouble)bin - 6.0;
}

void tyon_sensitivity_selector_set_from_rmp(TyonSensitivitySelector *selector, TyonRmp *rmp) {
	GaminggearXyScales *gaminggear_selector = GAMINGGEAR_XY_SCALES(selector);
	guint sync = tyon_rmp_get_xy_synchronous(rmp);
	gaminggear_xy_scales_set_linked(gaminggear_selector, sync == TYON_PROFILE_SETTINGS_ADVANCED_SENSITIVITY_OFF);
	if (sync == TYON_PROFILE_SETTINGS_ADVANCED_SENSITIVITY_ON) {
		gaminggear_xy_scales_set_x_value(gaminggear_selector, bin_to_scale(tyon_rmp_get_sensitivity_x(rmp)));
		gaminggear_xy_scales_set_y_value(gaminggear_selector, bin_to_scale(tyon_rmp_get_sensitivity_y(rmp)));
	} else
		gaminggear_xy_scales_set_x_value(gaminggear_selector, bin_to_scale(tyon_rmp_get_sensitivity(rmp)));
}

void tyon_sensitivity_selector_update_rmp(TyonSensitivitySelector *selector, TyonRmp *rmp) {
	GaminggearXyScales *gaminggear_selector = GAMINGGEAR_XY_SCALES(selector);
	guint sync = gaminggear_xy_scales_get_linked(gaminggear_selector) ? TYON_PROFILE_SETTINGS_ADVANCED_SENSITIVITY_OFF : TYON_PROFILE_SETTINGS_ADVANCED_SENSITIVITY_ON;
	tyon_rmp_set_xy_synchronous(rmp, sync);
	if (sync == TYON_PROFILE_SETTINGS_ADVANCED_SENSITIVITY_ON) {
		tyon_rmp_set_sensitivity_x(rmp, scale_to_bin(gaminggear_xy_scales_get_x_value(gaminggear_selector)));
		tyon_rmp_set_sensitivity_y(rmp, scale_to_bin(gaminggear_xy_scales_get_y_value(gaminggear_selector)));
	} else
		tyon_rmp_set_sensitivity(rmp, scale_to_bin(gaminggear_xy_scales_get_x_value(gaminggear_selector)));
}

GtkWidget *tyon_sensitivity_selector_new(void) {
	TyonSensitivitySelector *selector;

	selector = TYON_SENSITIVITY_SELECTOR(g_object_new(TYON_SENSITIVITY_SELECTOR_TYPE,
			"min", (gdouble)bin_to_scale(TYON_PROFILE_SETTINGS_SENSITIVITY_MIN),
			"max", (gdouble)bin_to_scale(TYON_PROFILE_SETTINGS_SENSITIVITY_MAX),
			"step", (gdouble)1.0,
			NULL));

	return GTK_WIDGET(selector);
}

static void tyon_sensitivity_selector_init(TyonSensitivitySelector *selector) {}

static void tyon_sensitivity_selector_class_init(TyonSensitivitySelectorClass *klass) {}
